#ifndef _AUDIO_PORT_H_ 
#define _AUDIO_PORT_H_ 1

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#ifdef LINUX
   #include <sys/types.h>
   #include <sys/stat.h>
   #include <fcntl.h>
   #include <sys/ioctl.h>

   #ifdef ALSA
      #include <sys/soundcard.h>
   #endif /* ALSA */
   #ifdef OSS
      #include <sys/soundcard.h>
/*    #if ((SOUND_VERSION > 360) && (defined(OSS_SYSINFO))) */
	  #if defined(SOUND_PCM_WRITE_CHANNELS)
         #define NEW_OSS
		 #define USE_CARD_CHANS
      #endif
      int get_card_max_chans(int fd);
   #endif /* OSS */

   #define AUDIO_DATUM_SIZE         2             /* bytes per sample */
   #define NUM_FRAGMENTS            10
   #define BASE_SND_DEVICE          "/dev/dsp"
   #ifdef MONO_DEVICES
      #define OUT_DEV_OFFSET        0
      #define IN_DEV_OFFSET         8
   #endif /* MONO_DEVICES */

   /* Some cards/drivers turn on SPDIF sync only when writing to device;
      16384 is enough to prevent glitching at start of playback.
   */
   #define ZERO_FRAMES_BEFORE       0
   #define ZERO_FRAMES_AFTER        16384

   int open_ports(int in_nchans, int max_inports, int in_ports[],
                  int out_nchans, int max_outports, int out_ports[],
                  int verbose, float srate, int nfrags, int *nframes);

#endif /* LINUX */

#ifdef MACOSX
   #include <CoreAudio/CoreAudio.h>

   #define NUM_FRAGMENTS            2
   #define ZERO_FRAMES_BEFORE       0
   #define ZERO_FRAMES_AFTER        16384

   int open_macosx_ports(int in_nchans, int out_nchans,
                           AudioDeviceID *iport, AudioDeviceID *oport,
                           int verbose, float srate, int nfrags, int *nframes);
   void macosx_audio_read(void);
   void macosx_audio_write(void);
   void macosx_cmixplay_audio_write(short *buf);
#endif /* MACOSX */

#ifdef SGI
   #include <dmedia/audio.h>
   #include <ulocks.h>

   #define ZERO_FRAMES_BEFORE       0
   #define ZERO_FRAMES_AFTER        16384

   /* This macro is only in AL 2.0, so I use it to distinguish 2.0 from 1.0 */
   #ifdef AL_MAX_STRLEN
      #define AL_2_0
   #endif

   #ifndef AL_2_0
      #define alSetSampFmt ALsetsampfmt
      #define alSetFloatMax ALsetfloatmax
      #define alSetWidth ALsetwidth
      #define alSetChannels ALsetchannels
      #define alOpenPort ALopenport
      #define alClosePort ALcloseport
      #define alSetFloatMax ALsetfloatmax
      #define alNewConfig ALnewconfig
      #define alFreeConfig ALfreeconfig
      #define alSetConfig ALsetconfig
      #define alSetQueueSize ALsetqueuesize

      /* this function doesn't exist under 1.0 */
      const char *alGetErrorString(int err) { return ""; }
   #endif /* !AL_2_0 */

   int open_sgi_ports(char *out_port_str, int out_nchans, ALport *oport,
                      int verbose, float srate, int *nframes);

#endif /* SGI */

#ifdef __cplusplus
} /* extern "C" */
#endif
#endif /* _AUDIO_PORT_H_ */
