#include <ugens.h>
#include <sfheader.h>
#include <stdio.h>

/* these are all defined in sys/sound.c */
extern int      sfd[NFILES];
extern int      pointer[NFILES];
extern char     peakoff[NFILES];
extern int      bufsize[NFILES];
extern char     *sndbuf[NFILES];
extern char     *peak[NFILES];
extern char     *peakloc[NFILES];
extern SFHEADER sfdesc[NFILES];

/* a cmix routine to surgically alter segments of one or more channels in
 * a soundfile.   Arguments are 
 *	          p[0]	starting time;
 *		  p[1]  duration;
 *		  p[2]  overall amp;
 *	          p[3]	alter channel 0?  1 if yes, 0 if no;
 *	          p[4]	"     "       1?  " "  "    " "  "
 *		  p[5]  "     "       2?  " "  "    " "  "
 *		  p[6]	"     "       3?  " "  "    " "  "
*/
static float *amptable, tabs[2];

static void bmultout(float amp, int *chlist, int fno, int size);


double
alter(float p[], int n_args)   
{
	long nsamps,i;
	int chlist[4],j,counter,non,skip,unit;
	float amp;

	unit = 1;
	nsamps = setnote(p[0],p[1],unit);
	counter = 0;
	skip = SR/(float)resetval;

	amptable = floc(1);
	if (amptable) {
		int amplen = fsize(1);
		tableset(p[1], amplen, tabs);
	}
	else
		advise("alter", "Setting phrase curve to all 1's.");

	for(j=0,non=0;j<sfchans(&sfdesc[unit]);j++) { 
		chlist[j] = (int)p[j+3];
		if (chlist[j])
			non++;
		if ((j+3) >= n_args)
			chlist[j] = 0;
	}

	if((non == 0) || (non > sfchans(&sfdesc[unit])))
		die("alter", "Something wrong with this card, check args.");

	for (i=nsamps; i>0; i -= skip) {
		counter = i>skip? skip : i;
		if (amptable)
			amp = tablei(nsamps-i, amptable, tabs) * p[2];
		else
			amp = p[2];
		bmultout(amp,chlist,unit,counter*sfchans(&sfdesc[unit]));
	}
	endnote(unit);

   return 0.0;
}


static void
bmultout(float amp, int *chlist, int fno, int size)
{
	int i,j;
	short *ibuf;
	float *fbuf;
	int todo,remains;
	int nchans;
	int len = bufsize[fno]; 

	nchans = sfchans(&sfdesc[fno]);

refill:	todo = ((pointer[fno] + size) > len) 
				? len - pointer[fno] : size;
	if(sfclass(&sfdesc[fno]) == SF_SHORT) {
		for(i=0,ibuf = (short *)sndbuf[fno] + pointer[fno];i<todo;
								i += nchans)
			for(j=0; j<nchans; j++,ibuf++)
				if(chlist[j]) *ibuf *= amp;	
	}
	
	else {
		for(i=0,fbuf = (float *)sndbuf[fno] + pointer[fno];i<todo;
								i += nchans)
			for(j=0; j<nchans; j++,fbuf++)
				if(chlist[j]) *fbuf *= amp;
	}
	pointer[fno] += todo;

	if(pointer[fno] == len) {
		_backup(fno);
		if(!peakoff[fno])
			_chkpeak(fno);
		_writeit(fno);
		_readit(fno);
		pointer[fno] = 0;
	}

	if(size -= todo) goto refill;
}

