#include <ugens.h>
#include <sfheader.h>
#include <stdio.h>
#include <stdlib.h>

/* panecho: stereo panning delay instrument
*
*  p0 = output start time
*  p1 = input start time
*  p2 = duration
*  p3 = amplitude multiplier
*  p4 = channel 0 delay time
*  p5 = channel 1 delay time
*  p6 = regeneration multiplier (< 1!)
*  p7 = ring-down duration
*  p8 = input channel number <optional>
*  assumes function slot 1 is the amplitude envelope
*/

#define ARRSIZE 16384
extern SFHEADER      sfdesc[NFILES];


double panecho(p,n_args)
float p[];
int n_args;
{
	long i,nsamps,n,m,count,smps = 0;
	long delsamps;
	float amp,val,wait1,wait2,regen;
	float *delarray1,*delarray2;
	int deltabs1[2],deltabs2[2],j;
	float in[2*ARRSIZE],out[2*ARRSIZE];
	float *amparr,amptabs[2];
	int inchans,outchans,inchan,skip;

	setnote(p[1],p[2],0);
	nsamps = setnote(p[0],p[2],1);

	inchans = sfchans(&sfdesc[0]);
	outchans = sfchans(&sfdesc[1]);
	if (outchans != 2) {
		fprintf(stderr,"output must be stereo!!!!!!!\n");
		return(-1);
		}

	delsamps = p[4] * SR + 0.5;
	if( (delarray1 = (float *)valloc((size_t)(delsamps * sizeof(float)))) == NULL ) {
		fprintf(stderr,"Sorry, Charlie -- no space\n");
		exit(-1);
		}
	wait1 = p[4];
	delset(delarray1,deltabs1,wait1);

	delsamps = p[5] * SR;
	if( (delarray2 = (float *)valloc((size_t)(delsamps * sizeof(float)))) == NULL ) {
		fprintf(stderr,"Sorry, Charlie -- no space\n");
		exit(-1);
		}
	wait2 = p[5];
	delset(delarray2,deltabs2,wait2);

	regen = p[6];

	amparr = floc(1);
	if (amparr) {
		int lenamp = fsize(1);
		tableset(p[2], lenamp, amptabs);
	}
	else
		advise("panecho", "Setting phrase curve to all 1's.");

	skip = SR/(float)resetval;
	j = 0;
	amp = p[3];
	inchan= 0;
	if (n_args > 8) inchan = p[8];
        if ((inchan+1) > inchans) {
		fprintf(stderr,"uh oh, you have asked for channel %d of a %d-channel file...\n",inchan,inchans);
		exit(-1);
		}
	for (i = nsamps; i > 0; i -= ARRSIZE) {
		count = (i < ARRSIZE) ? i : ARRSIZE;
		count *= inchans;
		bgetin(in,0,count);
		m = 0;
                for (n = 0; n < count; n += inchans) {
                        while (!j--) {
                                if (amparr)
                                   val = tablei(smps,amparr,amptabs) * amp;
                                else
                                   val = amp;
                                j = skip;
                                smps += skip;
                                }
			out[m] = (in[n+inchan]*val) + (delget(delarray2, wait2, deltabs2)*regen);
			out[m+1] = delget(delarray1, wait1, deltabs1);
			delput(out[m+1], delarray2, deltabs2);
			delput(out[m], delarray1, deltabs1);
			m += outchans;
			}
                baddout(out,1,count*2/inchans);
                }
	
	nsamps = p[7] * SR;
	for (i = 0; i < nsamps; i++) {
		out[0] = delget(delarray2, wait2, deltabs2) * regen;
		out[1] = delget(delarray1, wait1, deltabs1);
		ADDOUT(out,1);
		delput(out[1], delarray2, deltabs2);
		delput(out[0], delarray1, deltabs1);
		}
	endnote(1);
}


profile()
{
	UG_INTRO("panecho",panecho);
}

