#include <ugens.h>
#include <sfheader.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/file.h>
#include <sys/types.h>

/* vwavetable -- a vibrato wavetable sythesis instrument
*
*  p0 = start
*  p1 = duration
*  p2 = pitch (oct.pc or hz)
*  p3 = amplitude
*  p4 = vibrato rate low
*  p5 = vibrato rate high
*  p6 = vibrato depth (percentage)
*  p7 = stereo spread [optional]
*  assumes function table 1 is the synthesis waveform
*  assumes function table 2 is the amplitude envelope
*  assumes function table 3 is the vibrato waveform
*
*/

extern SFHEADER      sfdesc[NFILES];

double
vwavetable(float p[], int n_args)
{
	int i,j,k,nsamps;
	float val=0.,out[2],si,amp,phase,vamp;
	float *wavetable, *amptable, *vibtable;
	float amptabs[2];
	int lenwave, lenvib;
	float sibot,sitop,vsi,vsibot,vsitop,vsidiff,vphase,sidiff;
	int skip,chans;

/* p0=start, p1=dur, p2=pitch, p3=amp, p4=riset, p5=decayt, p6=viblo
	p7=vibhi, p8=seed, p9=vibdepth  */
/* assumes function 1 is waveform and function 2 is envelope function */
/* function 3 is vibrato waveform */

	nsamps = setnote(p[0],p[1],1); /* set number samples, file position */
	chans = sfchans(&sfdesc[1]);

	if (p[2] < 15.0) 
		p[2] = cpspch(p[2]);

	wavetable = floc(1);
	if (wavetable == NULL)
		die("vwavetable", "You need to store a waveform in function 1.");
	lenwave = fsize(1);
	sibot = (p[2] - (0.01 * p[2])) * (float)lenwave/SR;
	sitop = (p[2] + (0.01 * p[2])) * (float)lenwave/SR;
	sidiff = sitop - sibot;

	amptable = floc(2);
	if (amptable) {
		int lenamp = fsize(2);
		tableset(p[1], lenamp, amptabs);
	}
	else
		advise("vwavetable", "Setting phrase curve to all 1's.");

	vibtable = floc(3);
	if (vibtable == NULL)
		die("vwavetable", "You haven't made the vibrato waveform (table 3).");
	lenvib = fsize(3);

	vsibot = p[4] * (float)lenvib/SR;
	vsitop = p[5] * (float)lenvib/SR;
	vsidiff = vsitop - vsibot;
	vsi = ((rrand()+1.0)/2.0) * vsidiff;
	vsi += vsibot;

	j = k = 0;
	skip = SR/(float)resetval;

	phase = 0.0;
	vphase = 0.0;
	amp = p[3];
	vamp = p[6];

	for(i = 0; i < nsamps; i++) {
		while (!j--) {
			if (amptable)
				val = amp * table(i, amptable, amptabs);
			else
				val = amp;
			j = skip;
			}

		while (!k--) {
			vsi = (( (rrand()+1.0)/2.0 ) * vsidiff) + vsibot;
//printf("vsi: %f\n",vsi);
			k = lenvib/vsi;
			}

		si = sibot + (oscil(vamp,vsi,vibtable,lenvib,&vphase) * sidiff);
		out[0] =  oscili(val,si,wavetable,lenwave,&phase);
		if (chans > 1) {
			out[1] = (1.0 - p[7]) * out[0];
			out[0] *= p[7];
			}

		ADDOUT(out,1);
		}
	endnote(1); /* close 'er up */

	return 0.0;
}


int profile()
{
	UG_INTRO("vwavetable",vwavetable);

	return 0;
}

