/* RTcmix  - Copyright (C) 2000  The RTcmix Development Team
   See ``AUTHORS'' for a list of contributors. See ``LICENSE'' for
   the license to this software and for a DISCLAIMER OF ALL WARRANTIES.
*/
/* rev'd for v2.3, JGG, 20-Feb-00 */

#include <stdio.h>
#include <unistd.h>
#include <assert.h>
#ifdef SGI
#include <dmedia/audio.h>
#endif
#include <globals.h>
#include <prototypes.h>
#include <buffers.h>
#include "../rtstuff/rtdefs.h"


static int dev_ibuf_allocated = 0;

#if defined(LINUX) || defined(SGI)
/* Array of input buffers for audio device, in format wanted by driver.
   If driver wants interleaved samps, use only dev_ibuf[0], which contains
   interleaved samples. Otherwise, each buffer in dev_ibuf contains one
   channel of samples.
*/
static IBufPtr dev_ibuf[MAXBUS];
#endif

#ifdef MACOSX
#include <audio_port.h>
#endif

/* local prototypes */
static void allocate_dev_ibuf(void);
static int read_from_audio_device(void);



/* ---------------------------------------------------- allocate_dev_ibuf --- */
/* Allocate either one interleaved buffer or multiple single buffers,
   depending on whether MONO_DEVICES is defined.
*/
static INLINE void
allocate_dev_ibuf()
{
#if defined(LINUX) || defined(SGI)
#ifdef MONO_DEVICES
   int i;

   for (i = 0; i < audioNCHANS; i++)
      dev_ibuf[i] = allocate_ibuf_ptr(RTBUFSAMPS);
#else /* !MONO_DEVICES */
   dev_ibuf[0] = allocate_ibuf_ptr(RTBUFSAMPS * NCHANS);
#endif /* !MONO_DEVICES */
#endif /* LINUX || SGI */
#ifdef MACOSX
   /* MACOSX doesn't use dev_ibuf */
   dev_ibuf_allocated = 1;
#endif /* MACOSX */
}


/* ----------------------------------------------- read_from_audio_device --- */
static INLINE int
read_from_audio_device()
{
   int   i, j, n, result;

#ifdef LINUX
 #ifdef MONO_DEVICES

   int nbytes = RTBUFSAMPS * sizeof(IBUFTYPE);

   for (n = 0; n < audioNCHANS; n++) {
      IBufPtr src = dev_ibuf[n];
      BufPtr dest = audioin_buffer[n];

      result = read(in_port[n], src, nbytes);
      if (result == -1)
         return -1;

      for (i = 0; i < RTBUFSAMPS; i++)
         dest[i] = (BUFTYPE) src[i];
   }

 #else /* !MONO_DEVICES */

   int nbytes = RTBUFSAMPS * audioNCHANS * sizeof(IBUFTYPE);
   IBufPtr src = dev_ibuf[0];

   result = read(in_port[0], src, nbytes);
   if (result == -1)
      return -1;

   for (n = 0; n < audioNCHANS; n++) {
      BufPtr dest = audioin_buffer[n];

      for (i = 0, j = n; i < RTBUFSAMPS; i++, j += audioNCHANS)
         dest[i] = (BUFTYPE) src[j];
   }

 #endif /* !MONO_DEVICES */
#endif /* LINUX */

#ifdef MACOSX
   macosx_audio_read();
   result = 0;
#endif /* MACOSX */

#ifdef SGI

   int nsamps = RTBUFSAMPS * audioNCHANS;
   IBufPtr src = dev_ibuf[0];

   result = ALreadsamps(in_port, src, nsamps);
   if (result == -1)
      return -1;

   for (n = 0; n < audioNCHANS; n++) {
      BufPtr dest = audioin_buffer[n];

      for (i = 0, j = n; i < RTBUFSAMPS; i++, j += audioNCHANS)
         dest[i] = (BUFTYPE) src[j];
   }

#endif /* SGI */

   return 0;
}


/* ----------------------------------------------------------- rtgetsamps --- */
void
rtgetsamps()
{
   int   err;

   assert(audio_on == 1);

   if (!dev_ibuf_allocated) {
	   allocate_dev_ibuf();
	   dev_ibuf_allocated = 1;
   }
   err = read_from_audio_device();
   if (err)
      fprintf(stderr, "rtgetsamps: bad read from audio device\n");
}


