#include <stdio.h>
#include <linux/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <math.h>
#include <sys/stat.h>
#include "../H/sfheader.h"
#include "../H/byte_routines.h"

#define BUFSIZE 8192

/* soundfile descriptors */
int  sfd;            
int status;
SFHEADER      sfdesc;
SFMAXAMP      sfm;
struct stat   sfst;
int swap;

int big_endian;

int check_endian() {
  int i = 1;
  
  if(*(char *)&i) 
    return 0; /* Little endian */
  else
    return 1; /* Big endian */
}

void
main (int argc, char* argv[]) 
{
  int len,format,stereo,speed,cycle,frag_size,samps,srate,chans,magic,class;
  int Nspeed,Nmagic,Nchans,Nclass;
  int audio_fd, out_port;
  int arg,i,j,c1,c2;
  int ifp, ifp2;
  SFHEADER sfh;
  short sampbuff[BUFSIZE];
  float time;
  int check;
  int rec_time;

#ifdef USE_SNDLIB

  fprintf(stderr, "Sorry -- not working yet with sndlib\n");
  exit(1);

#else /* !USE_SNDLIB */

  /* Check % of args */
  if (argc != 3) {
    fprintf(stderr, "Usage:  sfrecord <sfname> <time>\n");
    exit(1);
  }

  if (argc == 3) {
    rec_time = atoi(argv[2]);
    printf("Recording for %f seconds\n",rec_time);
  }

  printf("======== RTCmix (record) LINUX ========\n");
  
  big_endian = check_endian();
  if (big_endian) {
    printf("This is a Big Endian machine\n");
  }
  else {
    printf("This is a Little Endian machine\n");
  }

  status = 2;
  rwopensf(argv[1],sfd,sfdesc,sfst,"newplay",i,status);
  
  if (sfdesc.sfinfo.NeXTheader.magic != 0) {
    if (swap) {
      byte_reverse4(&sfdesc.sfinfo.NeXTheader.magic);
      byte_reverse4(&sfdesc.sfinfo.NeXTheader.dataLocation);
      byte_reverse4(&sfdesc.sfinfo.NeXTheader.dataFormat);
      byte_reverse4(&sfdesc.sfinfo.NeXTheader.dataSize);
      byte_reverse4(&sfdesc.sfinfo.NeXTheader.samplingRate);
      byte_reverse4(&sfdesc.sfinfo.NeXTheader.channelCount);
    }
    printf("OLD NEXT DATA ============\n");
    printf("magic = %d\n",sfdesc.sfinfo.NeXTheader.magic);
    printf("dataLocation = %d\n",sfdesc.sfinfo.NeXTheader.dataLocation);
    printf("dataFormat = %d\n",sfdesc.sfinfo.NeXTheader.dataFormat);
    printf("dataSize = %d\n",sfdesc.sfinfo.NeXTheader.dataSize);
    printf("samplingRate = %d\n",sfdesc.sfinfo.NeXTheader.samplingRate);
    printf("channelCount = %d\n",sfdesc.sfinfo.NeXTheader.channelCount);
    printf("==========================\n");
  }
    

  speed = sfsrate(&sfdesc);
  chans = sfchans(&sfdesc);
  magic = sfmagic(&sfdesc);
  class = sfclass(&sfdesc);

  /* Print out some stuff for folks */
  printf("File:  %s\n",argv[1]);
  printf("rate: %d\n", speed);
  printf("chans: %d\n", chans);
  printf("magic #:  %d\n", magic);
  if (class == SF_SHORT)
    printf("sfclass:  SHORT\n");
  else if (class == SF_FLOAT) 
    printf("sfclass:  FLOAT\n");
  else
    printf("sfclass:  UNKOWN\n");

  /* Open the audio port */
  if ((audio_fd = (int)open("/dev/dsp", O_RDONLY, 0)) == -1) { 
    perror("/dev/dsp");
    exit(1);
  }

  /* Set the fragment size */
  arg = 0x777f000f;
  if (ioctl(audio_fd, SNDCTL_DSP_SETFRAGMENT, &arg)==-1) {
    perror("FRAGMENT SIZE");
    exit(1);
  }

  /* Put the audio port in "sync" mode */
  /* ioctl(audio_fd, SNDCTL_DSP_SYNC, 0); */

  /* Set the format for output samples */
  format = AFMT_S16_LE;
  if (ioctl(audio_fd, SNDCTL_DSP_SETFMT, &format)==-1) { 
    perror("SNDCTL_DSP_SETFMT");
    exit(1);
  }

  /* Set stereo / mono (1/0) */
  stereo = chans-1;
  if (ioctl(audio_fd, SNDCTL_DSP_STEREO, &stereo)==-1) { 
    perror("SNDCTL_DSP_STEREO");
    exit(1);
  }

  /* Set sample rate */
  if (ioctl(audio_fd, SNDCTL_DSP_SPEED, &speed)==-1)
    { /* Fatal error */
      perror("SNDCTL_DSP_SPEED");
      exit(1);
    }

  /* Get the size of the audio buffer */
  /* Note that the driver computes this optimally */
  if (ioctl(audio_fd, SNDCTL_DSP_GETBLKSIZE, &frag_size) == -1) {
    exit(1);
  }
  printf("Audio buffer:  %d\n",frag_size);

  time = 0;
  check = 1;

  printf("Time:  ");

  if (class != SF_SHORT) {
    fprintf(stderr,"Sorry, I can only record integer soundfiles\n");
    close(audio_fd);
    exit(1);
  }

  while (time < rec_time) {
    samps = read(audio_fd, sampbuff, 2*BUFSIZE);
    if (samps < 0) {
      printf("%d samples read\n",samps);
      perror("AUDIO READ");
      exit(1);
    }
    if (swap) {
      /* swap the bytes ... if necessary */
      for (i=0;i<BUFSIZE;i++) {
	byte_reverse2(&sampbuff[i]);
      }
    }
    if ((len = write(sfd, sampbuff, 2*BUFSIZE)) == -1) {
      perror("FILE WRITE");
      exit(1);
    }
    
    time += (float) BUFSIZE/speed;
    if (time > check) {
      fprintf(stdout,"%d ",check);
      check++;
    }
    fflush(stdout);
    
    /* zero the buffer again ... there might be leftover garbage at the last buffer */
    for (i=0;i<BUFSIZE;i++) {
      sampbuff[i] = 0;
    }
  }

  for (i=0;i<BUFSIZE;i++) {
    sampbuff[i] = 0;
  }
  if ((len = write(sfd, sampbuff, 2*BUFSIZE)) == -1) {
    perror("FILE WRITE");
    exit(1);
  }

  /* Close the audio port */
  close(audio_fd);
  /* Close the audio file */
  close(ifp);

  printf("\n");

#endif /* !USE_SNDLIB */

}




